import dlib
import colorsys

import numpy as np
import cv2 as cv

#from kmeans_pytorch import kmeans
from sklearn.cluster import KMeans

class ImageProcessor:
    detector = dlib.get_frontal_face_detector()
    
    lower = np.array([0, 133, 77], dtype = np.uint8)
    upper = np.array([255, 173, 127], dtype = np.uint8)
    
    def __init__(self):
        pass
    
    def image_get_face(self, image):
        result = list()
        
        try:
            faces = self.detector(image)
            for f in faces:
                l, t, r, b = f.left(), f.top(), f.right(), f.bottom()
                result.append(image[t:b, l:r])
        except Exception as e:
            print(e)
        finally:
            pass
        
        return result
    
    def image_get_skin(self, image):
        
        ycrcb = cv.cvtColor(image, cv.COLOR_BGR2YCrCb)
        mask = cv.inRange(ycrcb, self.lower, self.upper)
        masked = cv.bitwise_and(image, image, mask=mask)
        
        return masked
       
    def image_get_kmeans(self, image, clusters, types="RGB"):
        result = list()
        
        image = cv.cvtColor(image, cv.COLOR_BGR2RGB)
        
        kmeans = KMeans(n_clusters=clusters)
        cluster = kmeans.fit(image.reshape(-1, 3))
        
        #cluster_ids_x, cluster_centers = kmeans(X=torch.from_numpy(image.reshape(-1, 3)), num_clusters=clusters, distance='euclidean', tol=1e-2 ,device=torch.device('cuda:0'))
        
        for centers in cluster.cluster_centers_:
            if(types=="RGB"):
                result.append(centers)
            elif(types=="HSV"):
                (h, s, v) = colorsys.rgb_to_hsv(centers[0]/255, centers[1]/255, centers[2]/255)
                color = [h * 255, s * 255, v * 255]
                result.append(color)
            else:
                pass
        
        return np.array(result)